# frozen_string_literal: true

require "bundler/gem_tasks"
require "rake/testtask"
require "rake/clean"
require "yard"

CLEAN << FileList.new("doc/*") do |list|
  list.exclude /.*\.md/
  list.include "tmp/*"
end

Rake::TestTask.new :test do |t|
  t.libs << "lib"
  t.libs << "spec"
  t.test_files = FileList["spec/**/*_spec.rb"]
  # Supress `circular require` error messages from minitest
  t.warning = false
end

task spec: :test
task default: :spec

desc "Runs the linter"
task :lint do
  sh "bundle exec rubocop"
end

desc "Generates documentation"
task :doc do
  sh "bundle exec yard"
end
